<?php

namespace Teknisa\SM\Util\Procedure;

use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class FctCalcModpag {
    
    const NOME = "FCT_CALC_MODPAG";
    protected $connection;
    
    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }
    
    /**
     * Executa a procedure.
     * 
     * @param integer $nrorg
     * @param string  $dtreferencia
     * @param string  $cdfilial
     * @param string  $cdmodpgto
     * @param string  $cdcliente
     * @param string  $idfaturacontr
     * 
     * @return mixed[]
     */
    public function executar($nrorg, $dtreferencia, $cdfilial, $cdmodpgto, $cdcliente, $idfaturacontr) {
        $procedure = new StoredProcedure($this->connection, self::NOME);
        
        //Entrada
        $procedure->addParam(new Param("P_NRORG",          Param::PARAM_INPUT,   $nrorg,           Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param("P_DTREFERENCIA",   Param::PARAM_INPUT,   $dtreferencia,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIAL",       Param::PARAM_INPUT,   $cdfilial,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDMODPGTO",      Param::PARAM_INPUT,   $cdmodpgto,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDCLIENTE",      Param::PARAM_INPUT,   $cdcliente,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDFATURACONTR",  Param::PARAM_INPUT,   $idfaturacontr,   Param::PARAM_TYPE_STR));
        
        //Saida
        $procedure->addParam(new Param("P_INDIAS",           Param::PARAM_OUTPUT,   null,   Param::PARAM_TYPE_INT, 50));
        $procedure->addParam(new Param("P_DTBASE",           Param::PARAM_OUTPUT,   null,   Param::PARAM_TYPE_STR, 50));
        $procedure->addParam(new Param("P_IDFORMAPGTO",      Param::PARAM_OUTPUT,   null,   Param::PARAM_TYPE_STR, 50));
        $procedure->addParam(new Param("P_IDTIPODTPGT",      Param::PARAM_OUTPUT,   null,   Param::PARAM_TYPE_STR, 50));
        $procedure->addParam(new Param("P_IDTPPERIODO",      Param::PARAM_OUTPUT,   null,   Param::PARAM_TYPE_STR, 50));
        $procedure->addParam(new Param("P_NRPARCELAS",       Param::PARAM_OUTPUT,   null,   Param::PARAM_TYPE_STR, 50));
        $procedure->addParam(new Param("P_NRPARCPGTO",       Param::PARAM_OUTPUT,   null,   Param::PARAM_TYPE_STR, 50));
        $procedure->addParam(new Param("P_CDFORMPGTO",       Param::PARAM_OUTPUT,   null,   Param::PARAM_TYPE_STR, 50));
        $procedure->addParam(new Param("P_IDVALORPARCELAS",  Param::PARAM_OUTPUT,   null,   Param::PARAM_TYPE_STR, 50));
        $procedure->addParam(new Param("P_BOCONSFERIBANC",   Param::PARAM_OUTPUT,   null,   Param::PARAM_TYPE_STR, 50));
        
        $retorno = $procedure->execute();
        $data = $retorno["P_DTBASE"];
        $retorno["P_DTBASE"] = explode(" ", $data)[0];
        return $retorno;
    }
    
}

?>